%%
%%     BASIC SYLLABLE STRUCTURE THEORY (P&S, chap 8)
%%
%%         includes proper as well as hacked ons (onsitb)
%%
%%  constraints:  ons, onsitb (hacked, doesn't apply phrase-initially)
%%    parse, fillnuc, fillons, nocod
%%    marg(X), nuc(X), where X = lovowel/hivowel/liquid/nasal/vdfrid/
%%                                 vlfric/vdstop/vlstop
%%
%%  segments:  a,i,u,r,l,m,n,z,s,f,x,hh,b,d,g,t,k,kw
%%
%%   see itb.rnk for a sample ranking, for Imdlawn Tashliyt Berber
%%

verbosity(0).

sonority(a,lovowel).

sonority(i,hivowel).
sonority(u,hivowel).

sonority(r,liquid).
sonority(l,liquid).

sonority(m,nasal).
sonority(n,nasal).

sonority(z,vdfric).

sonority(s,vlfric).
sonority(f,vlfric).
sonority(x,vlfric).
sonority(hh, vlfric).  % pharyngeal

sonority(b,vdstop).
sonority(d,vdstop).
sonority(g,vdstop).

sonority(t,vlstop).
sonority(k,vlstop).
sonority(kw,vlstop).


aux_names([]).


adv(N,Seg,N, [Seg], [parse]).              % no contig implementation

adv(0,Seg,2,[ons(Seg)],[marg(Son)]) :-
  sonority(Seg,Son).   % ordinary onset

sit(0,2,[ons(nil)],[fillmarg]).

sit(0,2,[],[ons]).                        % no onset

adv(1,Seg,2,[ons(Seg)],[marg(Son)]) :-
  sonority(Seg,Son).   % ordinary onset

sit(1,2,[ons(nil)],[fillmarg]).

sit(1,2,[],[ons,onsitb]).                        % no onset


adv(2,Seg,3,[nuc(Seg)],[peak(Son)]) :-
   sonority(Seg,Son).

sit(2,3,[nuc(nil)],[fillnuc]).
  
sit(3,1,[],[]).                  % next syllable, no coda

adv(3,Seg,1,[cod(Seg)],[nocod,marg(Son)]) :-
  sonority(Seg,Son).


sit(0,end,[],[]).                % OK to end
sit(1,end,[],[]).                % ditto

%%  Structure Display.  Converts the output list to a more attractive
%%   format - here just reversing it (internal rep is last-processed-first).

typography(In,Out) :-
  reverse(In,Out).
 