%%
%%                       Generator for Galilean (=Grossly oversimplfied)
%%                            Syllable Theory, P&S Chapter 6.
%%
%%  Avilable Constraints (marks):
%%
%%    fill, nuc, ons, nocod, fillnuc, fillons, nocod, contig
%%
%%      in a ranking, use either `fill', or `fillons' and `fillnuc'
%%      (only the ones used in the ranking will count for evaluation)   
%%      `contig' militates against non-parsing within a syllable.
%%
%%    to see why it's there, try the ranking
%%      fill nocod parse ons
%%    with the input
%%      a b d a
%%    then compare with `contig' added to the ranking.
%%
%%  Available Segments and classes.  If you add a segment that contains
%%   anything other than lower case letters, surround them with quotes,
%%   e.g. conson('N').  Otherwise odd things may happen.

vowel(a).
vowel(e).
vowel(i).
vowel(o).
vowel(u).

conson(p).
conson(b).
conson(m).
conson(t).
conson(d).
conson(n).
conson(s).
conson(k).
conson(g).
conson(q).


%%  Automaton Definition
%%   sit-clauses define transitions where no input is consumed.
%%   adv-clauses define transitions where 2nd Arg is represent consumed
%%     input symbol (only one symbol at a time).
%%   the first number is the input state, the second the output state;
%%     these do not have to be numbers, but can be arbitrary prolog terms.
%%   2nd-to-last arg is list of output symbols ([] = empty list = no output).
%%   last arg is list of marks to be added for constraint violations.
%%   Cogent Prolog should be able to handle any standard prolog code you
%%     write for you own custom adv/sit predicates (but the results
%%     might overstress my OT engine!).

%% `auxiliary automata' (part of a try-hard implementation of
%%     Lardil, not need for GST)

aux_names([]).


%%  The real automaton.
%%

adv(0,Seg,0,[Seg],[parse]).          % skip, incurring a parse violation

adv(N,Seg,N,[Seg],[parse,contig]) :- % ditto, with contig also.
  not (N=0).  

adv(0,Seg,1,[ons(Seg)],[]) :-    % parse a consonant as onset, penalty-free.
  conson(Seg).

sit(0,1,[ons(nil)],[fill, fillons]).      % epenthetic onset

sit(0,1,[],[ons]).               % no onset

adv(1,Seg,2,[nuc(Seg)],[]) :-    % parse a vowel as nucleus, penalty-free.
 vowel(Seg).

sit(1,2,[nuc(nil)],[fill,fillnuc]).      % epenthetic nucleus
  
sit(2,0,[],[]).                  % next syllable, no coda

adv(2,Seg,0,[cod(Seg)],[nocod]):- % parse coda
  conson(Seg).

sit(0,end,[],[]).                % OK to end


%%  Structure Display.  Converts the output list to a more attractive
%%   format - here just reversing it (internal rep is last-processed-first).

typography(In,Out) :-
  reverse(In,Out).
