%verbosity_level(0).
%:-retract(show_ranking).

rank(R) :-
  retractall(ranking(_)),
  retractall(undominated(_)),
  rank2(R).

rank2([H|T]) :-
  list(H),
  dominators(H),
  assert(ranking(T)).

rank2(R) :-
  assert(ranking(R)).

dominators([H|T]) :-
  assert(undominated(H)),
  dominators(T).

dominators([]).

write_ranking(R) :-
  show_ranking,!,
  write('Ranking:'),nl,
  write_ranking2(R).

write_ranking(_).

write_ranking2([H|R]) :-
  list(H),!,
  write('  Inviolable: '),
  write_list(H),nl,
  write('  '),write_list(R),nl.

write_ranking2(R) :-
  write('  '),
  write_list(R),nl.

write_list([H|T]) :-
  write(H),write(' '),
  write_list(T).

write_list([]).
  

verbosity(N) :-
  retractall(verbosity_level(_)),
  assert(verbosity_level(N)).

message(parse_noranking,_) :-
  write('No ranking, enter ranking in rank window'),nl.

message(remaining_input,[Seg,Rest]) :-
  verbosity_level(N),
  N>0,!,
  write('Remaining Input: '),write(Seg),write('...'),write(Rest),nl.

message(curr_cand,[[_|Input],Candidates]) :- 
  progress_report(Input),
  verbosity_level(N),
  N>0,!,
  write('Viable Candidates:'),nl,
  writeres(Candidates),nl.

message(init_cand,[Input,Candidates]) :- 
  verbosity_level(N),
  N>0,!,
%  write('Initial Input: '),write(Input),nl,
  write('Initial Candidates: '),nl,
  writeres(Candidates),nl.

message(final_cand,Configs) :-
  write('Final Output: '),nl,
  writeout(Configs),nl.


message(init_pool,Configs) :-
  verbosity_level(2),!,
  write('Initial Candidates: '),nl,
  writeres(Configs),nl.


message(worse_score,[Cand,Stand]) :-
  verbosity_level(2),!,
  write(' Rejecting: '),writeres([Cand]).



message(_,_).

/*
continue :-
  verbosity_level(N),
  N>0,!,
  write('Continue (y/n)? '),
  get(C),
  continue(C).
*/

%%  defined as C-predicate
% continue.  

/*
continue(121) :- !.

continue(_) :-
  write('Generation Aborted'),nl,nl,
  fail.  
*/


writeres([]) :- !.
writeres([res(N,Out,Score)|Rest]) :-
  writeres(N,Out,Score),nl,
  writeres(Rest).
writeres(N,Out,Score) :-
  typography(Out,Out2),
  write(Out2),
  write('  / '),write(N),
  write('  / '), write(Score).

writeout([]) :- !.
writeout([res(_,Out,Score)|Rest]) :-
  writeout(Out,Score),
  writeout(Rest).
writeout(Out,Score) :-
  typography(Out,Out2),
  write(' '),
  write(Out2),nl,
  write('  Marks: '),
  ranking(Ranking),
  writescore(Ranking,Score),nl.

writescore([],[]) :- !.
writescore([Mark|RestRank],[Score|RestScore]) :-
  writescore2(Mark,Score),
  writescore(RestRank,RestScore).

writescore2(_,0) :- !.
writescore2(M,S) :-
 write(' '),write(M),write('/'),write(S).
