%%%    OT Structure Generator Predicates - 1 front version
%%        support predicates in otsa.pl


initial_config(Input,res(0:A0,[],Score),Ranking) :-
  create_score(Ranking,Score),
  aux_names(Names),
  aux_inits(Names,A0).

auxiliaries(MState,Output,AStates0,Marks0,AStates,Marks) :-
  aux_names(ANames),
  auxiliaries(ANames,MState,Output,AStates0,Marks0,AStates,Marks).

auxiliaries([AName|RestNames],MState,Output,[AState0|RestStates0],
    Marks0,[AState|RestStates],Marks) :-
  aux(AName,MState,Output,AState0,AState,AMarks),
  append(AMarks,Marks0,Marks1),
  auxiliaries(RestNames,MState,Output,RestStates0,Marks1,RestStates,Marks).

auxiliaries([],_,_,_,M,[],M).


sit_link1(Input,Configs,Newbie,Oldies,Ranking) :- 
  member(res(N0:A0,Out0,Score0),Configs),
  sit(N0,N,Output,Marks0),
  end_ok(Input,N),
  auxiliaries(N,Output,A0,Marks0,A,Marks),
  not(hopeless(Marks)),
  update_score(Score0,Marks,Score,Ranking),
  append(Output,Out0,Out),
  Newbie=res(N:A,Out,Score),
  not(sux(Newbie,Oldies,Ranking)).

end_ok(I,end) :- !,
  I=[].
end_ok(_,_).

sit_link(Input,Configs0,Configs,Oldies,Ranking) :-
   findall(Config,sit_link1(Input,Configs0,Config,Oldies,Ranking), Configs).

sitting(_,[],OutCands,OutCands,_) :- !.  % No more candidates, return output.

sitting(Input,InCands,OutCands0,OutCands,Ranking) :-
  sit_link(Input,InCands,NewCands0,OutCands0,Ranking),
  cull(NewCands0,NewCands0,NewCands1,Ranking), % zap worst newbies
  cull(OutCands0,NewCands1,OutCands1,Ranking), % zap oldies worse than newbies
  append(NewCands1,OutCands1,OutCands2),       % combine survivers 
  sitting(Input,NewCands1,OutCands2,OutCands,Ranking).


adv_link1(Seg,Configs,res(N:A,Out,Score),Ranking) :-
  member(res(N0:A0,Out0,Score0),Configs),
  adv(N0,Seg,N,Output,Marks0),
  auxiliaries(N,Output,A0,Marks0,A,Marks),
  not(hopeless(Marks)),
  update_score(Score0,Marks,Score,Ranking),
  append(Output,Out0,Out).

adv_link(Seg,Configs0,Configs,Ranking) :-
   findall(Config,adv_link1(Seg,Configs0,Config,Ranking), Configs).

stepping(Seg,Rest,InCands,OutCands,Ranking) :-
  adv_link(Seg,InCands,OutCands0,Ranking),
%  message(init_pool,OutCands0),
  cull(OutCands0,OutCands0,OutCands1,Ranking),
  sitting(Rest,OutCands1,OutCands1,OutCands2,Ranking),
  duplicates(OutCands2,OutCands).


aux_inits([_|Names],[0|Rest]) :-
  aux_inits(Names,Rest).
aux_inits([],[]).


parse([]) :-
  write('Nothing to do -- see ya later'),nl.

parse(Input) :-
  retractall(input_string(_)),
  retractall(input_length(_)),
  length(Input,L),
  assert(input_string(Input)), % progress-report into
  assert(input_length(L)),   % progress-report info
  ranking(R),!,
  findall(Config,initial_config(Input,Config,R),Configs0),
  cull(Configs0,Configs0,Configs1,R),
  sitting(Input,Configs1,Configs1,Configs,R),
  message(init_cand,[Input,Configs]),
  parse(Input,Configs,R).

parse(_) :-
  message(parse_noranking,[]).

parse([],Configs0,_) :-
  finalists(Configs0,Configs),
  message(final_cand,Configs).


parse([Seg|Rest],Configs0,R) :-
  continue,!,
  message(remaining_input,[Seg,Rest]),
  stepping(Seg,Rest,Configs0,Configs1,R),
  message(curr_cand,[[Seg|Rest],Configs1]),
  parse(Rest,Configs1,R).
  
parse(_,_,_).

hopeless(Marks) :-
  member(Mark,Marks),
  undominated(Mark).

cull(Configs,[],Configs,_) :- !. % no DV, don't bother

% remove from arg1 the configs that suck w.r.t. the configs in arg2
cull([],_,[],_) :- !.
cull([First|Rest],DV,Configs,Ranking) :-
  sux(First,DV,Ranking),!,
  cull(Rest,DV,Configs,Ranking).
cull([First|Rest],DV,[First|Configs],Ranking) :-
  cull(Rest,DV,Configs,Ranking).


sux(Item, Items, Ranking) :- %
  member(Item0,Items),
  Item0=res(N,_,Score0),
  Item=res(N,_,Score),
  not(Score == Score0),  % can't be worse if you're the same!
  worse_score(Score,Score0),
  message(worse_score,[Item,Item0]),!.


duplicates([],[]).
duplicates([H|Rest0],Out) :-
  duplicate(H,Rest0),!,
  duplicates(Rest0,Out).
duplicates([H|Rest],[H|Out]) :-
  duplicates(Rest,Out).

duplicate(res(N,Struc,_),[res(N,Struc,_)|_]) :- !.
duplicate(H,[_|T]) :-
  duplicate(H,T).

finalists([],[]) :- !.
finalists([First|Rest0],[First|Rest]) :-
  First=res(end:A,_,_),
  final_aux(A),
  finalists(Rest0,Rest).
finalists([_|Rest0],Rest1) :-
  finalists(Rest0,Rest1).

final_aux([]).
final_aux([end|A]) :-
  final_aux(A).

create_score([H|R],[0|S]) :-
  create_score(R,S).
create_score([],[]).

count_mark(Mark,[Mark|Rest],List,Number) :- !,
  count_mark(Mark,Rest,List,N0),
  Number is N0+1.
count_mark(Mark,[Head|Rest0],[Head|Rest],Number) :- !,
  count_mark(Mark,Rest0,Rest,Number).
count_mark(_,[],[],0).
  

update_score([Head0|Rest0],Marks,[Head|Rest],[Mark|RestRank]) :- !,
  count_mark(Mark,Marks,RestMarks,Number),
  Head is Head0+Number,
  update_score(Rest0,RestMarks,Rest,RestRank).

update_score([],_,[],[]).

worse_score([N|Rest0],[N|Rest]) :- !,
  worse_score(Rest0,Rest).
worse_score([N|_],[M|_]) :-
  N>M.
